<?php

namespace Litecms\Affiliate\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Affiliate\Http\Requests\AffiliateRequest;
use Litecms\Affiliate\Interfaces\AffiliateRepositoryInterface;
use Litecms\Affiliate\Models\Affiliate;

/**
 * Resource controller class for affiliate.
 */
class AffiliateResourceController extends BaseController
{

    /**
     * Initialize affiliate resource controller.
     *
     * @param type AffiliateRepositoryInterface $affiliate
     *
     * @return null
     */
    public function __construct(AffiliateRepositoryInterface $affiliate)
    {
        parent::__construct();
        $this->repository = $affiliate;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Affiliate\Repositories\Criteria\AffiliateResourceCriteria::class);
    }

    /**
     * Display a list of affiliate.
     *
     * @return Response
     */
    public function index(AffiliateRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Affiliate\Repositories\Presenter\AffiliatePresenter::class)
                ->$function();
        }

        $affiliates = $this->repository->paginate();

        return $this->response->title(trans('affiliate::affiliate.names'))
            ->view('affiliate::affiliate.index', true)
            ->data(compact('affiliates'))
            ->output();
    }

    /**
     * Display affiliate.
     *
     * @param Request $request
     * @param Model   $affiliate
     *
     * @return Response
     */
    public function show(AffiliateRequest $request, Affiliate $affiliate)
    {

        if ($affiliate->exists) {
            $view = 'affiliate::affiliate.show';
        } else {
            $view = 'affiliate::affiliate.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('affiliate::affiliate.name'))
            ->data(compact('affiliate'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new affiliate.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AffiliateRequest $request)
    {

        $affiliate = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('affiliate::affiliate.name')) 
            ->view('affiliate::affiliate.create', true) 
            ->data(compact('affiliate'))
            ->output();
    }

    /**
     * Create new affiliate.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AffiliateRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $affiliate                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('affiliate::affiliate.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('affiliate/affiliate/' . $affiliate->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/affiliate/affiliate'))
                ->redirect();
        }

    }

    /**
     * Show affiliate for editing.
     *
     * @param Request $request
     * @param Model   $affiliate
     *
     * @return Response
     */
    public function edit(AffiliateRequest $request, Affiliate $affiliate)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('affiliate::affiliate.name'))
            ->view('affiliate::affiliate.edit', true)
            ->data(compact('affiliate'))
            ->output();
    }

    /**
     * Update the affiliate.
     *
     * @param Request $request
     * @param Model   $affiliate
     *
     * @return Response
     */
    public function update(AffiliateRequest $request, Affiliate $affiliate)
    {
        try {
            $attributes = $request->all();

            $affiliate->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('affiliate::affiliate.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('affiliate/affiliate/' . $affiliate->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('affiliate/affiliate/' . $affiliate->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the affiliate.
     *
     * @param Model   $affiliate
     *
     * @return Response
     */
    public function destroy(AffiliateRequest $request, Affiliate $affiliate)
    {
        try {

            $affiliate->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('affiliate::affiliate.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('affiliate/affiliate/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('affiliate/affiliate/' . $affiliate->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple affiliate.
     *
     * @param Model   $affiliate
     *
     * @return Response
     */
    public function delete(AffiliateRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('affiliate::affiliate.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('affiliate/affiliate'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/affiliate/affiliate'))
                ->redirect();
        }

    }

    /**
     * Restore deleted affiliates.
     *
     * @param Model   $affiliate
     *
     * @return Response
     */
    public function restore(AffiliateRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('affiliate::affiliate.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/affiliate/affiliate'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/affiliate/affiliate/'))
                ->redirect();
        }

    }

}
